﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [StorableClass]
  public abstract class AlgorithmCharacteristicCalculator<TAlgorithm> : CharacteristicCalculator
    where TAlgorithm : class, IAlgorithm {

    public override IProblem Problem {
      get { return base.Problem; }
      set {
        base.Problem = value;
        Algorithm.Problem = value;
      }
    }

    [Storable]
    private ValueParameter<TAlgorithm> algorithmParameter;
    public IValueParameter<TAlgorithm> AlgorithmParameter { get { return algorithmParameter; } }
    public TAlgorithm Algorithm {
      get { return algorithmParameter.Value; }
      set { algorithmParameter.Value = value; }
    }
    
    [StorableConstructor]
    protected AlgorithmCharacteristicCalculator(bool deserializing) : base(deserializing) { }
    protected AlgorithmCharacteristicCalculator(AlgorithmCharacteristicCalculator<TAlgorithm> original, Cloner cloner)
    : base(original, cloner) {
      algorithmParameter = cloner.Clone(original.algorithmParameter);
    }
    protected AlgorithmCharacteristicCalculator() {
      Parameters.Add(algorithmParameter = new ValueParameter<TAlgorithm>("Algorithm", "The algorithm that calculates characteristics."));
    }
    protected AlgorithmCharacteristicCalculator(TAlgorithm algorithm) {
      Parameters.Add(algorithmParameter = new ValueParameter<TAlgorithm>("Algorithm", "The algorithm that calculates characteristics.", algorithm));
    }

    public override bool CanCalculate() {
      return Algorithm != null
        && Problem != null
        && Algorithm.ProblemType.IsAssignableFrom(Problem.GetType());
    }

    public override IEnumerable<IResult> Calculate() {
      Algorithm.Prepare(true);
      Algorithm.Start();
      
      foreach (var p in characteristics.CheckedItems) {
        yield return new Result(p.Value.Value, Algorithm.Results[p.Value.Value].Value);
      }
      Algorithm.Prepare(true);
    }
  }
}
